#include "ReloadController.h"

@implementation ReloadController


- (void) dealloc
{
  RELEASE(data);
  RELEASE(wc);
  [super dealloc];
}

- (id) init
{
  if( (self = [super init]) )
    {
      data = [[NSMutableArray alloc] initWithArray:
        [[NSDictionary dictionaryWithContentsOfFile: 
          [[NSBundle bundleForClass: [self class]] pathForResource: @"data" ofType: @"plist"]] 
          objectForKey: @"data"]];
      wc = [[NSWindowController alloc] initWithWindowNibName: @"Window" owner: self];
      [[wc window] makeKeyAndOrderFront: self]; 
      [outlineView setAutoresizesAllColumnsToFit: YES];
    }
  return self;
}

-(void) restart
{
  [[wc window] makeKeyAndOrderFront: nil];
}

- (void) reloadSelectedItem: (id) sender
{
  if( [outlineView selectedRow] > -1 )
    {
      [self _log: [NSString stringWithFormat: @"reloading row %i - %@\nreoloading children: %@",
        [outlineView selectedRow], [[outlineView itemAtRow: [outlineView selectedRow]] description],
        [childrenFlag intValue] ? @"YES" : @"NO"]];
      [outlineView reloadItem: [outlineView itemAtRow: [outlineView selectedRow]]
        reloadChildren: [childrenFlag intValue]];  
    }
}

- (void) reloadOutline: (id) sender
{
  RELEASE(data);
  data = [[NSMutableArray alloc] initWithArray:
        [[NSDictionary dictionaryWithContentsOfFile: 
          [[NSBundle bundleForClass: [self class]] pathForResource: @"data" ofType: @"plist"]] 
          objectForKey: @"data"]];
  [outlineView reloadData];
  [outlineView deselectAll: self];
}

- (void) resetLog: (id) sender
{
  [textView setString: @""];
}

- (id) outlineView: (NSOutlineView *) outlineView
             child: (int) index
            ofItem: (id) item
{
  if( !item )
    {
      return [data objectAtIndex: index];
    }
  if( [item objectForKey: @"children"] != nil )
    {
      return [[item objectForKey: @"children"] objectAtIndex: index];
    }
  return nil;
}

- (BOOL) outlineView: (NSOutlineView *) outlineView
    isItemExpandable: (id) item
{
  if( [[item objectForKey: @"children"] count] > 0 )
    return YES;
  return NO;
}

-    (int) outlineView: (NSOutlineView *) outlineView
numberOfChildrenOfItem: (id) item
{
  if( item == nil )
    return [data count];
  if( [item objectForKey: @"children"] )
    return [[item objectForKey: @"children"] count];
  return 0;
}

- (id)        outlineView: (NSOutlineView *) outlineView
objectValueForTableColumn: (NSTableColumn *) tableColumn
                   byItem: (id)item
{
  if( [[tableColumn identifier] isEqualToString: @"left"] )
    return [item objectForKey: @"left"];
  if( [[tableColumn identifier] isEqualToString: @"right"] )
    return [item objectForKey: @"right"];
  return nil;
}

- (void) outlineView: (NSOutlineView *) aOutlineView
     willDisplayCell: (id) aCell      
      forTableColumn: (NSTableColumn *) aTableColumn  
                item: (id) item
{
  [self _log: [NSString stringWithFormat: @"displaying item %@", [item description]]];
}

- (void) _log: (NSString *) theString
{
  NSMutableString *temp;

  temp = [[NSMutableString alloc] initWithString: [textView string]];
  [temp appendString: [NSString stringWithFormat: @"\n%@",theString]];
  [textView setString: temp];
  [textView scrollRangeToVisible:NSMakeRange([[textView string] length], 0)];
}

@end

    
