#include "DragDrop.h"

@implementation DragDrop


- (void) dealloc
{
  RELEASE(data);
  RELEASE(wc);
  RELEASE(tables);
  [super dealloc];
}

- (id) init
{
  NSMutableArray *dataOne;
  NSMutableArray *dataTwo;
  
  if( (self = [super init]) )
    {
      dataOne = [NSMutableArray arrayWithArray:
        [[NSDictionary dictionaryWithContentsOfFile: 
          [[NSBundle bundleForClass: [self class]] pathForResource: @"data" ofType: @"plist"]] 
        objectForKey: @"data"]];   
      dataTwo = AUTORELEASE([[NSMutableArray alloc] initWithArray: dataOne copyItems: YES]);
      data = [[NSMutableArray alloc] initWithObjects: @"spacer", dataOne, dataTwo, nil];
      
      wc = [[NSWindowController alloc] initWithWindowNibName: @"DragDrop" owner: self];
      [[wc window] makeKeyAndOrderFront: self]; 
      [tableViewOne setAutoresizesAllColumnsToFit: YES];
      [tableViewTwo setAutoresizesAllColumnsToFit: YES];
      [tableViewOne registerForDraggedTypes: [NSArray arrayWithObject: @"MyTableViewPboardType"]];
      [tableViewTwo registerForDraggedTypes: [NSArray arrayWithObject: @"MyTableViewPboardType"]];
      tables = [[NSArray alloc] initWithObjects: @"spacer", tableViewOne, tableViewTwo, nil];
    }
  return self;
}

-(void) restart
{
  [[wc window] makeKeyAndOrderFront: nil];
}

- (void) reloadTable: (id) sender
{
  int i = [sender tag];
  [data replaceObjectAtIndex: i withObject:
    [NSMutableArray arrayWithArray: [[NSDictionary dictionaryWithContentsOfFile: 
    [[NSBundle bundleForClass: [self class]] pathForResource: @"data" ofType: @"plist"]] 
    objectForKey: @"data"]]];
     
  [[tables objectAtIndex: i] reloadData];
  [[tables objectAtIndex: i] deselectAll: self];
}

- (void) resetLog: (id) sender
{
  [textView setString: @""];
}

- (void) delete: (id)sender
{
  [[data objectAtIndex: 2] removeAllObjects];
  [[tables objectAtIndex: 2] reloadData];
}

- (int) numberOfRowsInTableView: (NSTableView *) aTableView
{
  int i = [aTableView tag];
  return [[data objectAtIndex: i] count];
}

- (id)            tableView: (NSTableView *) aTableView
  objectValueForTableColumn: (NSTableColumn *) aTableColumn
                        row: (int) rowIndex
{
  NSMutableArray *d = [data objectAtIndex: [aTableView tag]];
  if( [[aTableColumn identifier] isEqualToString: @"left"] )
    return [[d objectAtIndex: rowIndex] objectForKey: @"left"];
  if( [[aTableColumn identifier] isEqualToString: @"right"] )
    return [[d objectAtIndex: rowIndex] objectForKey: @"right"];
  else
    return nil;
}

- (BOOL) tableView: (NSTableView *)tableView
         writeRows: (NSArray *)rows
      toPasteboard: (NSPasteboard *)pboard
{
  NSArray *d = [data objectAtIndex: [tableView tag]];
  NSMutableArray *ds = [NSMutableArray arrayWithCapacity: 1];
  NSEnumerator *e = [rows objectEnumerator];
  int row=0;
  
  while( (row = [[e nextObject] intValue]) )
    {
      [ds addObject: [d objectAtIndex: row]];
    }
  [pboard declareTypes: [NSArray arrayWithObject: @"MyTableViewPboardType"] owner: self];
  [pboard setPropertyList: [NSDictionary dictionaryWithObjectsAndKeys: ds, @"data", nil]  
    forType: @"MyTableViewPboardType"];
  [self _log: [NSString stringWithFormat: 
    @"%@ tableview writing to the pasteboard: %@", 
    [tableView tag]==1?@"left":@"right", [ds description]]];
  return YES;
}

- (NSDragOperation) tableView: (NSTableView *)tableView
                 validateDrop: (id<NSDraggingInfo>)info
                  proposedRow: (int)row
        proposedDropOperation: (NSTableViewDropOperation)operation
{
  if( [tableView isEqual: [info draggingSource]] )
    {
      [self _log: [NSString stringWithFormat: 
        @"%@ tableview rejecting the drag operation", [tableView tag]==1?@"left":@"right"]];  
      return NSDragOperationNone;
    }
  else
    {
      [self _log: [NSString stringWithFormat: 
        @"%@ tableview setting the drag operation to move", [tableView tag]==1?@"left":@"right"]]; 
      return NSDragOperationCopy; 
    }
}

- (BOOL) tableView: (NSTableView *)tableView
        acceptDrop: (id<NSDraggingInfo>)info
               row: (int)row
     dropOperation: (NSTableViewDropOperation)operation
{
  NSMutableArray *d = [data objectAtIndex: [tableView tag]];
  NSArray *ds;
  int k = row;
  int i = 0;
  
  ds = [[[info draggingPasteboard] propertyListForType: @"MyTableViewPboardType"] 
    objectForKey: @"data"];

  if( [ds count] > 0 )
    {
      [self _log: [NSString stringWithFormat: 
        @"%@ tableview accepting data: %@", [tableView tag]==1?@"left":@"right", [ds description]]];
      for( i=0; i < [ds count]; i++ )
        {
          [d insertObject: [ds objectAtIndex: i] atIndex: k];
          k++;          
        }
      [tableView reloadData];
      [[info draggingSource] reloadData];  
      return YES;  
    }
  return NO;
}

- (void) _log: (NSString *) theString
{
  NSMutableString *temp;

  temp = [[NSMutableString alloc] initWithString: [textView string]];
  [temp appendString: [NSString stringWithFormat: @"\n%@",theString]];
  [textView setString: temp];
  [textView scrollRangeToVisible:NSMakeRange([[textView string] length], 0)];
}

@end

    
