#include "Selection.h"

@implementation Selection


- (void) dealloc
{
  RELEASE(data);
  [super dealloc];
}

- (id) init
{
  if( (self = [super init]) )
    {
      data = [[NSMutableArray alloc] initWithArray:
        [[NSDictionary dictionaryWithContentsOfFile: 
          [[NSBundle bundleForClass: [self class]] pathForResource: @"data" ofType: @"plist"]] 
          objectForKey: @"data"]];
      wc = [[NSWindowController alloc] initWithWindowNibName: @"Selection" owner: self];
     [[wc window] makeKeyAndOrderFront: self]; 
     [tableView setAutoresizesAllColumnsToFit: YES];
    }
  return self;
}

-(void) restart
{
  [[wc window] makeKeyAndOrderFront: nil];
}

- (void) delete: (id) sender
{
  NSNumber *row;
  NSEnumerator *selectedRows;
  int i = 0;
  int x = 0;

  row = [NSNumber new];
  
  [self _log: [NSString stringWithFormat: @"%i rows selected", [tableView numberOfSelectedRows]]];
  
  if( [tableView numberOfSelectedRows] > 0 )
    {
      selectedRows = [tableView selectedRowEnumerator];
      while((row = [selectedRows nextObject]))
        {
          [self _log: [NSString stringWithFormat: 
            @"deleting: %@ (%i), %@", [row description], [row intValue]-i, 
            [[data objectAtIndex: [row intValue]-i] description]]];
          [data removeObjectAtIndex: [row intValue]-i];
          if( [row intValue] > x ) x = [row intValue];
          i++;
        }
        
      [tableView reloadData];
      [self _log: [NSString stringWithFormat: @"reloaded %i rows, selected row is %i", 
        [tableView numberOfRows], [tableView selectedRow]]]; 
  
      if( [tableView numberOfRows] > 0 ) 
        {
          x = x-i+1;
          if( x >= [tableView numberOfRows] ) x = x-1;
          [self _log: [NSString stringWithFormat: @"selecting row %i", x]]; 
          [tableView selectRow: x byExtendingSelection: NO];
        }  
    }  
}

- (void) resetTable: (id) sender
{
  RELEASE(data);
  data = [[NSMutableArray alloc] initWithArray:
        [[NSDictionary dictionaryWithContentsOfFile: 
          [[NSBundle bundleForClass: [self class]] pathForResource: @"data" ofType: @"plist"]] 
          objectForKey: @"data"]];
  [tableView reloadData];
  [tableView deselectAll: self];
}

- (void) resetLog: (id) sender
{
  [textView setString: @""];
}

- (int) numberOfRowsInTableView: (NSTableView *) aTableView
{
  return [data count];
}

- (id)            tableView: (NSTableView *) aTableView
  objectValueForTableColumn: (NSTableColumn *) aTableColumn
                        row: (int) rowIndex
{
  if( [[aTableColumn identifier] isEqualToString: @"left"] )
    return [[data objectAtIndex: rowIndex] objectForKey: @"left"];
  if( [[aTableColumn identifier] isEqualToString: @"right"] )
    return [[data objectAtIndex: rowIndex] objectForKey: @"right"];
  else
    return nil;
}

- (void) tableViewSelectionDidChange:(NSNotification *)aNotification
{ 
  if( [aNotification object] == tableView )
    {
      [self _log: [NSString stringWithFormat:
        @"numberOfSelectedRows reports %i rows selected.",
        [tableView numberOfSelectedRows]]];
      [self _log: [NSString stringWithFormat:
        @"selectedRowEnumerator reports %i rows selected.",
        [[[tableView selectedRowEnumerator] allObjects] count]]];  
    }
}

- (void) tableView: (NSTableView *)aTableView 
    setObjectValue: (id)anObject
    forTableColumn: (NSTableColumn *)aTableColumn 
               row: (int)rowIndex
{
  [self _log: @"entering tableView:setObjectValue:forTableColumn:row:"];
  if( [[aTableColumn identifier] isEqualToString: @"left"] )
      [[data objectAtIndex: rowIndex] setObject: anObject forKey: @"right"];
  else if( [[aTableColumn identifier] isEqualToString: @"right"] )
      [[data objectAtIndex: rowIndex] setObject: anObject forKey: @"right"];
}

- (void) _log: (NSString *) theString
{
  NSMutableString *temp;

  temp = [[NSMutableString alloc] initWithString: [textView string]];
  [temp appendString: [NSString stringWithFormat: @"\n%@",theString]];
  [textView setString: temp];
  [textView scrollRangeToVisible:NSMakeRange([[textView string] length], 0)];
}

@end

    
